# -*- coding: utf-8 -*-
# customchannels.py

import xbmcplugin,xbmcgui,sys,os,os.path,xbmcaddon,xbmc 
import xml.dom.minidom, shutil

__addonid__ = "plugin.video.mflive"
__addon__ = xbmcaddon.Addon(id=__addonid__)

CUSTOM_CHANNELS_DIR = xbmc.translatePath("special://home/customchannels")

class CustomChannels:
    
    def __init__(self):
        self.channels = {}
        self.LoadChannels()
    
    def LoadChannels(self):
        if not os.path.isdir(CUSTOM_CHANNELS_DIR):
            return
        for file in os.listdir(CUSTOM_CHANNELS_DIR):
            if os.path.isfile(os.path.join(CUSTOM_CHANNELS_DIR, file)):
                self.importChannels(os.path.join(CUSTOM_CHANNELS_DIR, file), file)
    
    def importChannels(self, channelsFile, source = ""):
        print "%s - import channels from file [%s]" %(__addonid__, channelsFile)
        if not os.path.isfile(channelsFile):
            return 0
        if source == "":
            source = os.path.split(channelsFile)[1]
        if source in self.channels:
            dialog = xbmcgui.Dialog()
            ret = dialog.yesno("提示", "您之前已经导入过同名文件，即将删除旧数据并导入新数据", "是否继续？")
            if not ret:
                return -1
        channelCount = 0
        channelCount = self.importMFTVChannels(channelsFile, source)
        if channelCount == 0:
            channelCount = self.importTogicOrVSTChannels(channelsFile, source)
        if channelCount > 0 and os.path.dirname(channelsFile) != CUSTOM_CHANNELS_DIR:
            if not os.path.exists(CUSTOM_CHANNELS_DIR):
                os.mkdir(CUSTOM_CHANNELS_DIR)
            try:
                shutil.copyfile(channelsFile, os.path.join(CUSTOM_CHANNELS_DIR, source))
            except Exception, e:
                print "%s - copy channels file [%s] failed" %(__addonid__, channelsFile), e
                channelCount = 0
        return channelCount
    
    def importMFTVChannels(self, channelsFile, source = ""):
        if not os.path.isfile(channelsFile):
            return 0
        if source == "":
            source = os.path.split(channelsFile)[1]
        channelCount = 0
        channels = {}
        try:
            doc = xml.dom.minidom.parse(channelsFile)
            root = doc.documentElement
            channel_nodes = root.getElementsByTagName("itemTV")
            for node in channel_nodes:
                url = ""
                urlNodes = node.getElementsByTagName('url')
                if len(urlNodes) > 0:
                    urlNode = urlNodes[0]
                    if urlNode.hasChildNodes():
                        url = urlNode.childNodes[0].data.encode('utf-8').strip()
                name = ""
                nameNodes = node.getElementsByTagName('channelName')
                if len(nameNodes) > 0:
                    nameNode = nameNodes[0]
                    if nameNode.hasChildNodes():
                        name = nameNode.childNodes[0].data.encode('utf-8').strip()
                icon = ""
                iconNodes = node.getElementsByTagName('pic')
                if len(iconNodes) > 0:
                    iconNode = iconNodes[0]
                    if iconNode.hasChildNodes():
                        icon = iconNode.childNodes[0].data.encode('utf-8').strip()
                if not name or not url:
                    continue
                if name in channels:
                    if not url in channels[name][0]:
                        #new url for an existing channel
                        channels[name][0].append(url)
                else:
                    urls = [url]
                    channels[name] = [urls, icon]
                    channelCount = channelCount + 1
        except:
            channelCount = 0
        if channelCount > 0:
            self.channels[source] = channels
        return channelCount
    
    def importTogicOrVSTChannels(self, channelsFile, source = ""):
        if not os.path.isfile(channelsFile):
            return 0
        if source == "":
            source = os.path.split(channelsFile)[1]
        channelCount = 0
        channels = {}
        try:
            doc = xml.dom.minidom.parse(channelsFile)
            if doc:
                return channelCount
        except:
            pass
        file = open(channelsFile,'r')
        try:
            channelList = file.readlines()
            for channel in channelList:
                channel = channel.strip()
                ch = channel.split(' ')
                if not len(ch)>1 or not ch[0] or not ch[len(ch)-1]:
                    ch = channel.split(',')
                if not len(ch)>1 or not ch[0] or not ch[len(ch)-1]:
                    continue
                name = ch[0].strip(' ,')
                url = ch[len(ch)-1].strip(' ,')
                if not name or not url:
                    continue
                if name in channels:
                    if not url in channels[name][0]:
                        #new url for an existing channel
                        channels[name][0].append(url)
                else:
                    urls = [url]
                    icon = ''
                    channels[name] = [urls, icon]
                    channelCount = channelCount + 1
        except:
            channelCount = 0
        finally:
            file.close()
        if channelCount > 0:
            self.channels[source] = channels
        return channelCount
    
    def getSources(self):
        return self.channels.keys()
    
    def getChannelsBySource(self, source):
        if source in self.channels:
            return self.channels[source]
        return {}
    
    def getChannelUrl(self, channelName):
        urls = []
        for source in self.getSources():
            channels = self.channels[source]
            if channelName in channels.keys():
                urlList = channels[channelName][0]
                for url in urlList:
                    urls.append(url)
        return urls
    
    def removeChannelSource(self, source):
        removed = 0
        if not source in self.channels:
            removed = 1
            return removed
        del self.channels[source]
        try:
            os.remove(os.path.join(CUSTOM_CHANNELS_DIR, source))
        except:
            removed = 2
            return removed
        return removed
    
customChannels = CustomChannels()