#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys
import os
import time
import traceback
import urllib
import json
from pkgutil import iter_modules
from xbmcswift2 import Plugin, actions, xbmc, xbmcgui
from urllib import quote_plus, unquote_plus, urlencode
import urllib2
import re
import xbmcvfs
import xbmcaddon
import subroutine
import user
import run

plugin = Plugin()
plugin_path = plugin.addon.getAddonInfo('path')
lib_path = os.path.join(plugin_path, 'lib')
sys.path[0:0] = [lib_path,
                 os.path.join(lib_path, 'poster-0.4-py2.6.egg'),
                 os.path.join(lib_path, 'rsa_x509_pem-0.1.0.egg')]

__addonid__ = "plugin.video.mfyun"
__addon__ = xbmcaddon.Addon(id=__addonid__)
RESOURCES_PATH = os.path.join(__addon__.getAddonInfo('path'), 'media', )
lib_path = os.path.join(__addon__.getAddonInfo('path'), 'lib' ,'modules')
sys.path.append (lib_path)
import utils
from client_api import ClientAPI, PCSApiError, ClientApiError, TRANSCODE_TYPES

@plugin.route('/settings/')
def settings():
    current_user = user.get_default_userid()

    items = [{
        'label': '当前帐号： [COLOR FFFFFF00]%s[/COLOR]' % (
            current_user or NULL_USER),
        'path': plugin.url_for('select_user'),
        'is_playable': True,
        'properties': {'isPlayable': ''}
    }, {
        'label': '帐号管理',
        'path': plugin.url_for('users_list')
    }, {
        'label': '清空缓存',
        'path': plugin.url_for('clear_cache'),
        'is_playable': True,
        'properties': {'isPlayable': ''}
    }]

    return items
        


@plugin.route('/list_user_dir/<userid>/<path>/')
def list_user_dir(userid, path):
    items = list_dir(userid, path)
    return plugin.finish(items, view_mode='thumbnail')

_api_maps = {}
NULL_USER = '[ 空 ]'


def get_uk(userid):
    userdata = plugin.get_storage('userdata')
    user = userdata.setdefault(userid, {})
    if 'uk' in user:
        return user['uk']

    uk = get_api(userid).get_uk()
    user['uk'] = uk
    userdata.sync()

    return uk

def creaet_api_with_publickey():
    api = ClientAPI()
    api.set_public_key(*_get_public_key())
    return api

@plugin.cached(360)
def _get_public_key():
    from simple_rsa import get_public_key
    cert = ClientAPI().get_cert()[0]
    return get_public_key(cert)


@plugin.cached(360)
def get_dlink(userid, path):
    api = get_api(userid)
    info = api.get_filemetas(path)['info']
    return info[0]['dlink']

def get_api(userid):
    global _api_maps
    if userid in _api_maps:
        return _api_maps[userid]

    api = creaet_api_with_publickey()
    userdata = plugin.get_storage('userdata')
    user = userdata.setdefault(userid, {})
    if 'session' in user:
        api.set_login_info(user['session'])

    _api_maps[userid] = api

    return api

@plugin.route('/delete_path/<userid>/<path>/')
def delete_path(userid, path):
    api = get_api(userid)
    try:
        plugin.notify('正在删除文件，请稍候...', delay=2000, image='DefaultIconInfo.png')
        api.delete(path)
    except PCSApiError as e:
        utils.show_msg(str(e), '')
    else:
        xbmc.sleep(1000)
        utils.refresh()

def delete_path_mf(userid, path):
    api = get_api(userid)
    try:
        api.delete(path)
    except PCSApiError as e:
        utils.show_msg(str(e), '')
    else:
        xbmc.sleep(1000)

def upload_file(userid, path):
    if not os.path.isfile(path):
        return
    api = get_api(userid)
    ret = api.upload_file(path, '/MoreFunTV/', overwrite='newcopy')
    return ret

def list_torrent(userid, path):
    if not userid:
        return []
    api = get_api(userid)
    ret = api.query_sinfo(path)
    file_sha1 = ret['torrent_info']['sha1']
    file_list = ret['torrent_info']['file_info']
    items = []
    for i in range(0,len(file_list)):
        file_name = file_list[i]['file_name']
        if is_video(file_name):
            items.append({
            'label' : file_name[file_name.rfind('/')+1:],
            'path' : 'plugin://plugin.video.mfyun/play_torrent/' + urllib.quote_plus(str(userid)) + '/' + urllib.quote_plus(path) + '/' + urllib.quote_plus(str(file_name)) + '/' + urllib.quote_plus(str(i+1)) + '/' + urllib.quote_plus(file_sha1) + '/'
            })
    return items

def list_shared_torrent(userid, path, uk, shareid):
    if not userid:
        return []
    api = get_api(userid)
    remote_path = '/MoreFunTV'
    try:
        ret = api.list_dir(remote_path)
    except ClientApiError as e:
        if e.obj['errno'] == -9:
            ret = api.create(remote_path)
    
    try:
        ret = api.transfer_share(shareid, uk, path, remote_path)
    except ClientApiError as e:
        if not e.obj['errno'] == 12 or not e.obj['info'][0]['errno'] == -30:
            return []
    torrent_path = remote_path + '/' + path[path.rfind('/')+1:]
    ret = api.query_sinfo(torrent_path)
    file_sha1 = ret['torrent_info']['sha1']
    file_list = ret['torrent_info']['file_info']
    items = []
    for i in range(0,len(file_list)):
        file_name = file_list[i]['file_name']
        if is_video(file_name):
            items.append({
            'label' : file_name[file_name.rfind('/')+1:],
            'path' : 'plugin://plugin.video.mfyun/play_torrent/' + urllib.quote_plus(str(userid)) + '/' + urllib.quote_plus(torrent_path) + '/' + urllib.quote_plus(str(file_name)) + '/' + urllib.quote_plus(str(i+1)) + '/' + urllib.quote_plus(file_sha1) + '/'
            })
    return items

def list_magnet(userid, path):
    if not userid:
        return []
    api = get_api(userid)
    ret = api.query_magnetinfo(path)
    file_list = ret['magnet_info']
    items = []
    for i in range(0,len(file_list)):
        file_name = file_list[i]['file_name']
        if is_video(file_name):
            items.append({
            'label' : file_name,
            'path' : 'plugin://plugin.video.mfyun/play_magnet/' + urllib.quote_plus(str(userid)) + '/' + urllib.quote_plus(path) + '/' + urllib.quote_plus(str(file_name)) + '/' + urllib.quote_plus(str(i+1)) + '/'
            })
    return items

def list_dir(userid=None, path='/'):
    if not userid:
        return []
    api = get_api(userid)
    entries = []
    for page in xrange(1, 21):
        tmp_entries = api.list_dir(path, page=page)
        entries.extend(tmp_entries)

        if len(tmp_entries) < 100:
            break

    dirs = []
    video_files = []
    torrent_files = []
    subtitle_files = []
    compresses_files = []
    other_files = []
    for entry in entries:
        filename = entry['server_filename']
        dlink = entry.get('dlink')
        file_md5 = entry.get('md5')
        imgurl = ''
        if 'thumbs' in entry and 'url1' in entry['thumbs']:
            imgurl = entry['thumbs']['url1'].replace(
                'size=c140_u90', 'size=c300_u300')

        if str(entry['isdir']) == '1' and filename != 'MoreFunTV':
            dirs.append(filename)
        elif is_video(filename):
            video_files.append((filename, dlink, imgurl, file_md5))
        elif is_torrent(filename):
            torrent_files.append((filename, dlink, imgurl, file_md5))
        elif is_subtitle(filename):
            subtitle_files.append((filename, dlink, imgurl, file_md5))
        #elif is_compressed(filename):
            #compresses_files.append((filename, dlink, imgurl, file_md5))
        else:
            other_files.append((filename, dlink, imgurl, file_md5))

    def get_context_menu(userid, path):
        if isinstance(path, unicode):
            path = path.encode('utf-8')

        return [(
            '[COLOR FFFFFF00]删除[/COLOR]',
            actions.background(
                plugin.url_for('delete_path', userid=userid, path=path))
        )]

    def get_subtitle(name):
        filename, _, ext = name.rpartition('.')
        for subfile, subfile_dlink, imgurl, md5 in subtitle_files:
            if (subfile.startswith(filename + '.')
                    and '.' not in subfile[len(filename) + 1:]):
                return subfile_dlink

        return ''

    items = [{
        'label': '[ %s ]' % name,
        'md5': '',
        'filepath': urllib.quote_plus(str((path + '/' + name).encode('utf-8'))),
        'path': plugin.url_for('list_user_dir', userid=userid,
                               path=(path + '/' + name).encode('utf-8')),
        'context_menu': get_context_menu(userid, path + '/' + name)
    } for name in dirs]

    if video_files:
        #plugin.set_setting('prefer_transcode', 'true')
        #prefer_transcode = plugin.get_setting('prefer_transcode', bool)
        items.extend([{
            'label': name,
            'md5': md5,
            'filepath': urllib.quote_plus(str((path + '/' + name).encode('utf-8'))),
            'path': ('plugin://plugin.video.mfyun/play_pan_video/' + urllib.quote_plus(str(userid)) + '/' + urllib.quote_plus(str((path + '/' + name).encode('utf-8'))) + '/' + urllib.quote_plus(str(name.encode('utf-8'))) + '/' + urllib.quote_plus(str(get_subtitle(name))) + '/' + str(md5) + '/'+ urllib.quote_plus(str((path + '/' + name).encode('utf-8'))) + '/'),
            'is_playable': True,
            'thumbnail': imgurl or api.get_thumbnail_url(path + '/' + name)
        } for name, dlink, imgurl, md5 in video_files])

    if torrent_files:
        items.extend([{
            'label': name,
            'md5': md5,
            'filepath': urllib.quote_plus(str((path + '/' + name).encode('utf-8'))),
            #'path': plugin.url_for('play_torrent', userid=userid, url=dlink),
            'path': 'plugin://plugin.video.mfyun/list_torrent/' + urllib.quote_plus(str(userid)) + '/' + urllib.quote_plus(str((path + '/' + name).encode('utf-8'))) + '/' + urllib.quote_plus(str(name.encode('utf-8'))) + '/' + str(md5) + '/'+ urllib.quote_plus(str((path + '/' + name).encode('utf-8'))) + '/',
            'is_playable': True,
            'properties': {'isPlayable': ''},
            'context_menu': get_context_menu(userid, path + '/' + name)
        } for name, dlink, imgurl, md5 in torrent_files])

    if compresses_files:
        items.extend([{
            'label': name,
            'md5': md5,
            'filepath': urllib.quote_plus(str((path + '/' + name).encode('utf-8'))),
            'path': plugin.url_for('show_compressed_file',
                                   name=name.encode('utf-8'),
                                   url=dlink),
            'context_menu': get_context_menu(userid, path + '/' + name)
        } for name, dlink, imgurl, md5 in compresses_files])

    if other_files and plugin.get_setting('show_non_video_files', bool):
        items.extend([{
            'label': name,
            'md5': md5,
            'filepath': urllib.quote_plus(str((path + '/' + name).encode('utf-8'))),
            'path': '',
            'thumbnail': imgurl,
            'is_playable': True,
            'properties': {'isPlayable': ''},
            'context_menu': get_context_menu(userid, path + '/' + name),
            'replace_context_menu': True
        } for name, dlink, imgurl, md5 in other_files])

    return items

_video_file_exts = None

def is_video(filename):
    return subroutine.is_video(filename)


def is_subtitle(filename):
    filename = filename.lower()
    return any(filename.endswith(ext) for ext in (
        '.srt', '.sub', '.ssa', '.smi', '.ass'))


def is_torrent(filename):
    return filename.lower().endswith('.torrent')




def list_mynotes(userid=None):
    if not userid:
        return []

    api = get_api(userid)
    categories = [c for c in api.list_note_categories()
                  if c['title'] == 'xbmc_bth']
    if not categories:
        res = api.add_category('xbmc_bth')
        category_id = res[0]['_key']
    else:
        category_id = categories[0]['_key']

    notes = api.list_note(category_id)
    if not notes:
        api.add_note('#示例\nhttp://pan.baidu.com/share/home?uk=791642990',
                     category_id)
        notes = api.list_note(category_id)

    items = []
    for note in notes:
        name = note['title']
        content = note['content']
        if content.startswith('#'):
            firstline, _, content = content.partition('\n')
            firstline = firstline.lstrip('#').strip()
            if firstline:
                name = firstline

        items.append({
            'label': name,
            'path': plugin.url_for(
                'show_note_btih', userid=userid, note_id=note['_key'],
                category_id=category_id),
            'is_playable': True,
            'properties': {'isPlayable': ''}
        })

    return items

def log(*args):
    plugin.log.info(args[0] if len(args) == 1 else args)


def quote_url(url):
    return quote_plus(url, '')


def is_compressed(filename):
    name = filename.lower()
    return name.endswith('.zip') or name.endswith('.rar')

reload(sys)
sys.setdefaultencoding('utf-8')


if __name__ == '__main__':
    plugin.run()
