#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys
import os
import time
import traceback
import urllib
import json
from pkgutil import iter_modules
from xbmcswift2 import Plugin, actions, xbmc, xbmcgui
from urllib import quote_plus, unquote_plus, urlencode
import urllib2
import re
import xbmcvfs
import xbmcaddon
import itvi_play

plugin = Plugin()
plugin_path = plugin.addon.getAddonInfo('path')
lib_path = os.path.join(plugin_path, 'resources', 'lib')
sys.path[0:0] = [lib_path,
                 os.path.join(lib_path, 'poster-0.4-py2.6.egg'),
                 os.path.join(lib_path, 'rsa_x509_pem-0.1.0.egg')]

__addonid__ = "plugin.video.mfyun"
__addon__ = xbmcaddon.Addon(id=__addonid__)
RESOURCES_PATH = os.path.join(__addon__.getAddonInfo('path'), 'media' )
lib_path = os.path.join(__addon__.getAddonInfo('path'), 'lib')
sys.path.append (lib_path)
import utils


def parse_baidupan_url(uk, shareid=None, fid=None, remote_dir=None,
                       bdclnd=None):
    items = []
    #sharid 不为 none，此时为非根目录
    if shareid:        
        n_shareitems = 0
        for page in xrange(1, 21):
            shareitems = list_baidu_share(shareid, uk, fid, remote_dir,
                                          page=page, bdclnd=bdclnd)                                         
            for share_item in shareitems:
                if str(share_item['isdir']) == '1':
                    items.append({
                                    'type'      :  'folder',
                                    'name'      :  share_item['server_filename'],
                                    'path'      :  share_item['path'],
                                    'filetype'  :  'folder',
                                    'asfolder'  :  True,
                                    'fs_id'     :  share_item['path'],
                                })
                else:
                    add_share_item(shareid,share_item, items)

            n_shareitems += len(shareitems)
            if len(shareitems) < 100:
                break
    else:
        #sharid 为 none，此时为根目录
        for share_item in list_baidu_share_home_all(uk):
            #如果是文件
            if share_item['feed_type'] == 'share' and not share_item.has_key('dir_cnt'):
                if is_useful_file(share_item['title']):
                    filetype = get_file_type(share_item['title'])
                    
                    items.append({
                                    'type'      : 'file',
                                    'name'      : share_item['filelist'][0]['server_filename'],
                                    'path'      : share_item['filelist'][0]['path'],
                                    'shareid'   : share_item['shareid'],
                                    'thumurl'   : share_item['filelist'][0]['thumburl'] if 'thumburl' in share_item['filelist'][0] else '',
                                    'filetype'  : filetype,
                                    'asfolder'  : True if filetype == 'torrent' or filetype == 'muf' else False,
                                    'fs_id'     : share_item['filelist'][0]['fs_id']
                                 })            
            #如果是文件夹
            elif share_item['feed_type'] == 'share' and share_item.has_key('dir_cnt'):
                path = ''
                for file in share_item['filelist']:
                    path += file['server_filename']+'\\'
                items.append({
                                'type'      :  'folder',
                                'name'      :  share_item['title'],
                                'path'      :  path,
                                'shareid'   :  share_item['shareid'],
                                'filetype'  :  'folder',
                                'asfolder'  :  True,
                                'fs_id'     :  ''
                             })
            #如果是专辑
            elif share_item['feed_type'] == 'album':         
                items.append({
                                'type'      :  'album',
                                'name'      :  share_item['title'],
                                'uk'        :  share_item['uk'],
                                'album_id'  :  share_item['album_id'],
                                'filetype'  :  'album',
                                'asfolder'  :  True,
                                'thumurl'   :  share_item['cover_thumb']
                             }) 
    return items

def list_baidu_share_home_all(uk):
    items = []
    for page in xrange(1, 21):
        shareitems = list_baidu_share_home(uk, page=page)
        items.extend(shareitems)
        if len(shareitems) < 100:
            break

    return items

def list_baidu_share_home(uk,page=1):
    start = (page-1)*100
    url = 'http://yun.baidu.com/pcloud/feed/getsharelist?&auth_type=1&request_location=share_home&start=%d&limit=100&query_uk=%s'%(start,uk)
    
    result = json.loads(utils.fetch_url(url))
    
    items = []
    if not result['errno']:
        items = result['records']

    return items
     
def list_baidu_share(shareid, uk, fid=None, remote_dir=None,
                     page=1, num=100, order='time', desc=1,
                     bdclnd=None):
    params = {'shareid': shareid, 'uk': uk}
    if fid:
        params['fid'] = fid
    elif remote_dir:
        params['dir'] = remote_dir
    else:
        params['root'] = 1
    if page > 1:
        params['page'] = page
    if 1 < num < 100:
        params['num'] = num
    if order != 'time':
        params['order'] = order
    if desc != 1:
        params['desc'] = desc

    url = 'http://pan.baidu.com/share/list?' + urlencode(params)
    
    
    #dir/path=%2Fvideos
    
    headers = {}
    if bdclnd:
        headers['Cookie'] = 'BDCLND=' + bdclnd
    shareitems = json.loads(utils.fetch_url(url, headers=headers))['list']
    return shareitems

def add_share_item(shareid,share_item, items, video_only=True, name_only=True):
    if video_only and not is_useful_file(share_item['server_filename']):
        return

    imgurl = ''
    if 'thumbs' in share_item and 'url1' in share_item['thumbs']:
        imgurl = share_item['thumbs']['url1'].replace(
            'size=c140_u90', 'size=c300_u300')

    name = share_item['server_filename'] if name_only else share_item['path'].rstrip('/')
    filetype = get_file_type(name)
    items.append({
                    'type'      : 'file',
                    'name'      : name, 
                    'path'      : share_item['path'],                   
                    'shareid'   : shareid,
                    'thumurl'   : imgurl,
                    'filetype'  : filetype,
                    'asfolder'  : True if filetype == 'torrent' or filetype == 'muf' else False,
                    'fs_id'     : share_item['fs_id'],
                })

def is_video(filename):
    _video_file_exts = [
        '.mkv', '.rmvb', '.rm', '.avi', '.mpg', '.mpeg', '.mp4',
        '.mov', '.3gp', '.asf', '.wmv', '.flv', '.ts']
    filename = filename.lower()
    return any(filename.endswith(ext) for ext in _video_file_exts)

def is_torrent(filename):
    return filename.lower().endswith('.torrent')

def is_zip(filename):
    return filename.lower().endswith('.zip')


def is_rar(filename):
    return filename.lower().endswith('.rar')

def is_compressed(filename):
    return is_zip(filename) or is_rar(filename)

def is_muf(filename):
    return filename.lower().endswith('.muf')

def is_useful_file(filename):
    return is_video(filename) or is_torrent(filename) or is_muf(filename)

def is_compressed_http_url(url):
    url = url.lower()
    return url.startswith('zip://http%3a%2f%2f') \
        or url.startswith('rar://http%3a%2f%2f')


def is_compressed_url(url):
    url = url.lower()
    return url.startswith('zip://') \
        or url.startswith('rar://')

def get_file_type(filename):
    filetype = ''
    if is_muf(filename):
        filetype = 'muf'
    elif is_video(filename):
        filetype = 'video'
    elif is_torrent(filename):
        filetype = 'torrent'
    else:
        filetype = 'other'
    
    return filetype